\\ Copyright 2014, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

atan2(y,x) = my(angle=arg(x+I*y)); if(angle<0,angle+2*Pi,angle);
deg(x) = x/180*Pi;

nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)!="t_VEC",
     if(! nearly_equal(x,y),
        return(Strprintf("different values\n%s\n%s",x,y)));
     return(0));
  if(length(x)!=length(y),
     return(Strprintf("different lengths %s %s",length(x),length(y))));
  for(i=1,#x,
     my(d=differ(x[i],y[i]));
     if(d != 0, return(Strprintf("at index %d %s",i,d))));
  return(0);
}

check_equal(got,want) =
{
  my(d = differ(got,want));
  if(d != 0,
     print("wrong\n",d); quit(1),
     print("ok"));
}

IMA(k) = 1/84 * (5*9^k + [-5,-3,5,3][(k%4)+1]*3^floor(k/2));
vector(8,k,my(k=k-1); IMA(k))

IMB(k) = 1/84 * (-4*9^k + [4,-6,-4,6][(k%4)+1]*3^floor(k/2));
vector(8,k,my(k=k-1); IMB(k))

IMC(k) = 1/168*(  6*9^k - 7*3^k + [1,9,-1,-9][(k%4)+1]*(3)^floor(k/2) );
vector(8,k,my(k=k-1); IMC(k))

\\---------------------------------------
\\ cos(a+30)^2 = (cos(a)*cos(30) - sin(a)*sin(30))^2
\\             = (sqrt(3)/2*cos(a) - 1/2*sin(a))^2
\\             = 3/4*cos(a)^2 - sqrt(3)/2*cos(a)*sin(a)^2 + 1/4*sin(a)^2
check_equal(vector(256,k,my(a=2*Pi/k); cos(a+deg(30))^2), \
            vector(256,k,my(a=2*Pi/k); 3/4*cos(a)^2 - sqrt(3)/2*cos(a)*sin(a) + 1/4*sin(a)^2));

\\ cos(a+30)*sin(a+30)
\\   = (cos(a)*cos(30) - sin(a)*sin(30))*(sin(a)*cos(30) + cos(a)*sin(30))
\\   = (sqrt(3)/2*cos(a) - 1/2*sin(a))*(sqrt(3)/2*sin(a) + 1/2*cos(a))
\\   = (sqrt(3)/2*cos(a) - 1/2*sin(a))*(sqrt(3)/2*sin(a) + 1/2*cos(a))
\\   = sqrt(3)/4*cos(a)^2 + 1/2*cos(a)*sin(a) - sqrt(3)/4*sin(a)^2
check_equal(vector(256,k,my(a=2*Pi/k); cos(a+deg(30))*sin(a+deg(30))), \
            vector(256,k,my(a=2*Pi/k); sqrt(3)/4*cos(a)^2 + 1/2*cos(a)*sin(a) - sqrt(3)/4*sin(a)^2));

\\ sin(a+30)^2
\\ = (sin(a)*cos(30) + cos(a)*sin(30))^2
\\ = (sqrt(3)/2*sin(a) + 1/2*cos(a))^2
\\ = 3/4*sin(a)^2 + 1/4*cos(a)^2 + sqrt(3)/2*sin(a)*cos(a)
check_equal(vector(256,k,my(a=2*Pi/k); sin(a+deg(30))^2), \
            vector(256,k,my(a=2*Pi/k); 3/4*sin(a)^2 + 1/4*cos(a)^2 + sqrt(3)/2*sin(a)*cos(a)))

\\---------------------------------------
\\ cos(a+120)^2
check_equal(vector(256,k,my(a=2*Pi/k); cos(a+deg(120))^2), \
            vector(256,k,my(a=2*Pi/k); 3/4*sin(a)^2 + 1/4*cos(a)^2 + sqrt(3)/2*sin(a)*cos(a)));

\\ cos(a+120)*sin(a+120)
check_equal(vector(256,k,my(a=2*Pi/k); cos(a+deg(120))*sin(a+deg(120))), \
            vector(256,k,my(a=2*Pi/k); -sqrt(3)/4*cos(a)^2 - 1/2*cos(a)*sin(a) + sqrt(3)/4*sin(a)^2));

\\ sin(a+120)^2
check_equal(vector(256,k,my(a=2*Pi/k); sin(a+deg(120))^2), \
            vector(256,k,my(a=2*Pi/k); 3/4*cos(a)^2 - sqrt(3)/2*cos(a)*sin(a) + 1/4*sin(a)^2));

\\---------------------------------------
\\ cos(a-30)^2
check_equal(vector(256,k,my(a=2*Pi/k); cos(a-deg(30))^2), \
            vector(256,k,my(a=2*Pi/k); 3/4*cos(a)^2 + sqrt(3)/2*sin(a)*cos(a) + 1/4*sin(a)^2))

\\ cos(a-30)*sin(a-30)
check_equal(vector(256,k,my(a=2*Pi/k); cos(a-deg(30))*sin(a-deg(30))), \
            vector(256,k,my(a=2*Pi/k); -sqrt(3)/4*cos(a)^2 + 1/2*cos(a)*sin(a) + sqrt(3)/4*sin(a)^2));

\\ sin(a-30)^2
\\ = (sin(a)*cos(30) - cos(a)*sin(30))^2
\\ = (sqrt(3)/2*sin(a) - 1/2*cos(a))^2
\\ = 1/4*cos(a)^2 - sqrt(3)/2*sin(a)*cos(a) + 3/4*sin(a)^2
check_equal(vector(256,k,my(a=2*Pi/k); sin(a-deg(30))^2), \
            vector(256,k,my(a=2*Pi/k); 1/4*cos(a)^2 - sqrt(3)/2*sin(a)*cos(a) + 3/4*sin(a)^2))

\\---------------------------------------
\\ I(alpha) = Ixx(k)*cos(a)^2 + Ixy(k)*sin(a)*cos(a) + Iyy(k)*sin(a)^2

I_ref(k,alpha) = IMA(k)*sin(alpha)^2 + IMB(k)*sin(alpha+deg(30))^2 + IMC(k);
Ia(k,a) = IMA(k)*sin(a)^2 + IMB(k)*sin(a+deg(30))^2 + IMC(k)*sin(a)^2 + IMC(k)*cos(a)^2;
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Ia(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_ref(k,a))))
Ia(k,a) = IMC(k)*cos(a)^2 + IMB(k)*sin(a+deg(30))^2 + (IMA(k) + IMC(k))*sin(a)^2;
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Ia(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_ref(k,a))))
Ia(k,a) = IMC(k)*cos(a)^2 + IMB(k)*(3/4*sin(a)^2 + 1/4*cos(a)^2 + sqrt(3)/2*sin(a)*cos(a)) + (IMA(k)+IMC(k))*sin(a)^2;
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Ia(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_ref(k,a))))
Ia(k,a) = (1/4*IMB(k) + IMC(k))*cos(a)^2 + sqrt(3)/2*IMB(k)*sin(a)*cos(a) + (IMA(k) + 3/4*IMB(k) + IMC(k))*sin(a)^2;
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Ia(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_ref(k,a))))
\\ Ixx(k) = 1/4*IMB(k) + IMC(k)
\\ Ixy(k) = sqrt(3)/2 * IMB(k)
\\ Iyy(k) = IMA(k) + 3/4*IMB(k) + IMC(k)

Ixx_ref(k) = 1/4*IMB(k) + IMC(k);
Ixx(k) = 1/4*1/84 * (-4*9^k + [4,-6,-4,6][(k%4)+1]*3^floor(k/2)) \
       + 1/168*(  6*9^k - 7*3^k + [1,9,-1,-9][(k%4)+1]*(3)^floor(k/2) );
check_equal(vector(256,k,my(k=k-1); Ixx(k)), \
            vector(256,k,my(k=k-1); Ixx_ref(k)))
Ixx(k) = 1/168 * (4*9^k - 7*3^k + [3,6,-3,-6][(k%4)+1]*3^floor(k/2));
check_equal(vector(256,k,my(k=k-1); Ixx(k)), \
            vector(256,k,my(k=k-1); Ixx_ref(k)))
Ixx(k) = 1/168 * (4*9^k - 7*3^k + [3,6][(k%2)+1]*(-3)^floor(k/2));
check_equal(vector(256,k,my(k=k-1); Ixx(k)), \
            vector(256,k,my(k=k-1); Ixx_ref(k)))
\\ vector(20,k,my(k=k-1); 8/168 * (4*9^k - 7*3^k + [3,6][(k%2)+1]*(-3)^floor(k/2)))
\\ 0, 1, 12, 129, 1224, 11169, 100980, 910305, 8197200, 73787841
\\ not in OEIS

Iyy_ref(k) = IMA(k) + 3/4*IMB(k) + IMC(k);
Iyy(k) = 1/84 * (5*9^k + [-5,-3,5,3][(k%4)+1]*3^floor(k/2)) \
       + 3/4*1/84 * (-4*9^k + [4,-6,-4,6][(k%4)+1]*3^floor(k/2)) \
       + 1/168*(  6*9^k - 7*3^k + [1,9,-1,-9][(k%4)+1]*(3)^floor(k/2) );
check_equal(vector(256,k,my(k=k-1); Iyy(k)), \
            vector(256,k,my(k=k-1); Iyy_ref(k)))
Iyy(k) = 1/168 * (10*9^k + 2*[-5,-3,5,3][(k%4)+1]*3^floor(k/2)) \
       + 1/168 * (3/2*-4*9^k + 3/2*[4,-6,-4,6][(k%4)+1]*3^floor(k/2)) \
       + 1/168 * (6*9^k - 7*3^k + [1,9,-1,-9][(k%4)+1]*(3)^floor(k/2) );
check_equal(vector(256,k,my(k=k-1); Iyy(k)), \
            vector(256,k,my(k=k-1); Iyy_ref(k)))
Iyy(k) = 1/168 * (10*9^k - 7*3^k + [-3, -6, 3, 6][(k%4)+1]*3^floor(k/2));
check_equal(vector(256,k,my(k=k-1); Iyy(k)), \
            vector(256,k,my(k=k-1); Iyy_ref(k)))
Iyy(k) = 1/168 * (10*9^k - 7*3^k - [3,6][(k%2)+1]*(-3)^floor(k/2));
check_equal(vector(256,k,my(k=k-1); Iyy(k)), \
            vector(256,k,my(k=k-1); Iyy_ref(k)))
\\ vector(20,k,my(k=k-1); 8/3/168 * (10*9^k - 7*3^k - [3,6][(k%2)+1]*(-3)^floor(k/2)))
\\ 0, 1, 12, 113, 1032, 9345, 84276, 758961, 6832080
\\ not in OEIS

Ixy_ref(k) = sqrt(3)/2 * IMB(k);
Ixy(k) = sqrt(3)/2 * 1/84 * (-4*9^k + [4,-6,-4,6][(k%4)+1]*3^floor(k/2));
check_equal(vector(256,k,my(k=k-1); Ixy(k)), \
            vector(256,k,my(k=k-1); Ixy_ref(k)))
Ixy(k) = sqrt(3)/168 * (-4*9^k + [4,-6][(k%2)+1]*(-3)^floor(k/2));
check_equal(vector(256,k,my(k=k-1); Ixy(k)), \
            vector(256,k,my(k=k-1); Ixy_ref(k)))
\\ vector(20,k,my(k=k-1); -4/168 * (-4*9^k + [4,-6][(k%2)+1]*(-3)^floor(k/2)))
\\ 0, 1, 8, 69, 624, 5625, 50616, 455517, 4099680, 36897201, 332074728
\\ not in OEIS

Ia(k,alpha) = Ixx(k)*cos(alpha)^2 + Ixy(k)*cos(alpha)*sin(alpha) + Iyy(k)*sin(alpha)^2;
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Ia(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_ref(k,a))))

\\------------------------------
Ia_plus_30(k,alpha) = Ixx(k)*cos(alpha+deg(30))^2 \
                     + Ixy(k)*cos(alpha+deg(30))*sin(alpha+deg(30)) \
                     + Iyy(k)*sin(alpha+deg(30))^2;
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Ia_plus_30(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_ref(k,a+deg(30)))))

Ia_plus_30(k,a) = (  Ixx(k)*(      3/4*cos(a)^2 - sqrt(3)/2*cos(a)*sin(a) +       1/4*sin(a)^2) \
                   + Ixy(k)*(sqrt(3)/4*cos(a)^2 +       1/2*cos(a)*sin(a) - sqrt(3)/4*sin(a)^2) \
                   + Iyy(k)*(      1/4*cos(a)^2 + sqrt(3)/2*sin(a)*cos(a) +       3/4*sin(a)^2));
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Ia_plus_30(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_ref(k,a+deg(30)))))

Ia_plus_30(k,a) = Ixx_plus_30(k)*cos(a)^2 + Ixy_plus_30(k)*cos(a)*sin(a) + Iyy_plus_30(k)*sin(a)^2;
Ixx_plus_30(k,a) =        3/4*Ixx(k) + sqrt(3)/4*Ixy(k) +       1/4*Iyy(k);
Ixy_plus_30(k,a) = -sqrt(3)/2*Ixx(k) +       1/2*Ixy(k) + sqrt(3)/2*Iyy(k);
Iyy_plus_30(k,a) =        1/4*Ixx(k) - sqrt(3)/4*Ixy(k) +       3/4*Iyy(k);
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Ia_plus_30(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_ref(k,a+deg(30)))))

Ixx_plus_30(k,a) =        3/4*1/168 * (4*9^k - 7*3^k + [3,6][(k%2)+1]*(-3)^floor(k/2)) \
                  + sqrt(3)/4*sqrt(3)/168 * (-4*9^k + [4,-6][(k%2)+1]*(-3)^floor(k/2)) \
                  +       1/4*1/168 * (10*9^k - 7*3^k - [3,6][(k%2)+1]*(-3)^floor(k/2));
Ixy_plus_30(k,a) = -sqrt(3)/2*1/168 * (4*9^k - 7*3^k + [3,6][(k%2)+1]*(-3)^floor(k/2)) \
                  +       1/2*sqrt(3)/168 * (-4*9^k + [4,-6][(k%2)+1]*(-3)^floor(k/2)) \
                  + sqrt(3)/2*1/168 * (10*9^k - 7*3^k - [3,6][(k%2)+1]*(-3)^floor(k/2));
Iyy_plus_30(k,a) =        1/4*1/168 * (4*9^k - 7*3^k + [3,6][(k%2)+1]*(-3)^floor(k/2)) \
                  - sqrt(3)/4*sqrt(3)/168 * (-4*9^k + [4,-6][(k%2)+1]*(-3)^floor(k/2)) \
                  +       3/4*1/168 * (10*9^k - 7*3^k - [3,6][(k%2)+1]*(-3)^floor(k/2));
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Ia_plus_30(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_ref(k,a+deg(30)))))

Ixx_plus_30(k,a) =             1/168 * ( 5/2*9^k - 7*3^k + [ 9/2, -3/2][(k%2)+1]*(-3)^floor(k/2));
Ixy_plus_30(k,a) =  1/2* sqrt(3)/168 * (   2*9^k         + [  -2,  -18][(k%2)+1]*(-3)^floor(k/2));
Iyy_plus_30(k,a) =             1/168 * (23/2*9^k - 7*3^k + [-9/2,  3/2][(k%2)+1]*(-3)^floor(k/2));
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Ia_plus_30(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_ref(k,a+deg(30)))))

\\------------------------------
Ia_minus_90(k,a) = Iyy(k)*cos(a)^2 - Ixy(k)*cos(a)*sin(a) + Ixx(k)*sin(a)^2;
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Ia_minus_90(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_ref(k,a-deg(90)))))

Ia_plus(k,a) = Iyy(k)*cos(a)^2 - Ixy(k)*cos(a)*sin(a) + Ixx(k)*sin(a)^2;
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Ia_plus(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_ref(k,a+deg(90)))))

\\------------------------------
I_next(k,a) = Ia_minus_90(k,a) \
             + 2*Ia_plus_30(k,a) \
             + 2*3^k * (1/2*sqrt(3)^k*sin(a-deg(30)))^2;
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_next(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_ref(k+1,a))))
I_next(k,a) = Ia_minus_90(k,a) \
             + 2*Ia_plus_30(k,a) \
             + 1/2*9^k * (1/4*cos(a)^2 - sqrt(3)/2*sin(a)*cos(a) + 3/4*sin(a)^2);
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_next(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); I_ref(k+1,a))))

\\------------------------------
\\ Ixx(k+1) =        3/2 * Ixx(k) + sqrt(3)/2 * Ixy(k) +       3/2 * Iyy(k) +       1/8*9^k
\\ Ixy(k+1) =   -sqrt(3) * Ixx(k)                      +   sqrt(3) * Iyy(k) - sqrt(3)/4*9^k
\\ Iyy(k+1) =        3/2 * Ixx(k) - sqrt(3)/2 * Ixy(k) +       3/2 * Iyy(k) +       3/8*9^k
check_equal(vector(256,k,my(k=k-1); 3/2 * Ixx(k) + sqrt(3)/2 * Ixy(k) +       3/2 * Iyy(k) + 1/8*9^k), \
            vector(256,k,my(k=k-1); Ixx(k+1)));
check_equal(vector(256,k,my(k=k-1); -sqrt(3)*Ixx(k)  + sqrt(3) * Iyy(k) - sqrt(3)/4*9^k), \
            vector(256,k,my(k=k-1); Ixy(k+1)));
check_equal(vector(256,k,my(k=k-1); 3/2 * Ixx(k) - sqrt(3)/2 * Ixy(k) +       3/2 * Iyy(k) + 3/8*9^k), \
            vector(256,k,my(k=k-1); Iyy(k+1)));
check_equal(Ixx(0), 0)
check_equal(Ixy(0), 0)
check_equal(Iyy(0), 0)
check_equal(Ixx(1), 1/8)
check_equal(Ixy(1), -sqrt(3)/4)
check_equal(Iyy(1), 3/8)
check_equal(     3/2 * Ixx(0) + sqrt(3)/2 * Ixy(0) +       3/2 * Iyy(0) +       1/8*9^0, \
            1/8)
check_equal(-sqrt(3) * Ixx(0)                      +   sqrt(3) * Iyy(0) - sqrt(3)/4*9^0, \
            -sqrt(3)/4)
check_equal(     3/2 * Ixx(0) - sqrt(3)/2 * Ixy(0) +       3/2 * Iyy(0) +       3/8*9^0, \
            3/8)
\\ with Iyy(0) = 1/12
Ixy1_with_Iyy1eq1 = -sqrt(3)*1/6;
check_equal(-sqrt(3) * Ixx(0)                      +   sqrt(3) * 1/12    - sqrt(3)/4*9^0, \
            Ixy1_with_Iyy1eq1)

\\ Ixx(k+1)-Iyy(k+1) = sqrt(3)*Ixy(k) - 1/4*9^k
check_equal(vector(256,k,my(k=k-1); sqrt(3)*Ixy(k) - 1/4*9^k), \
            vector(256,k,my(k=k-1); Ixx(k+1) - Iyy(k+1)));

\\ Ixy(k+2) = -sqrt(3) * (Ixx(k+1)-Iyy(k+1)) - sqrt(3)/4*9^(k+1)
check_equal(vector(256,k,my(k=k-1); -sqrt(3) * (Ixx(k+1) - Iyy(k+1))  - sqrt(3)/4*9^(k+1)), \
            vector(256,k,my(k=k-1); Ixy(k+2)));

check_equal(vector(256,k,my(k=k-1); -sqrt(3) * (sqrt(3)*Ixy(k) - 1/4*9^k)  - sqrt(3)/4*9^(k+1)), \
            vector(256,k,my(k=k-1); Ixy(k+2)));
check_equal(vector(256,k,my(k=k-1); -sqrt(3) * sqrt(3)*Ixy(k) + sqrt(3) * 1/4*9^k  - sqrt(3)/4*9^(k+1)), \
            vector(256,k,my(k=k-1); Ixy(k+2)));

\\ Ixy(k+2) = -3*Ixy(k) - 2*sqrt(3)*9^k
check_equal(vector(256,k,my(k=k-1); -3*Ixy(k) - 2*sqrt(3)*9^k), \
            vector(256,k,my(k=k-1); Ixy(k+2)));
\\ Ixy(k) = -3*Ixy(k-2) - 2*sqrt(3)*9^(k-2)
check_equal(vector(256,k,my(k=k-1); -3*Ixy(k-2) - 2*sqrt(3)*9^(k-2)), \
            vector(256,k,my(k=k-1); Ixy(k)));

Ixy_2k(h) = if(h<0,error("h<0"), h==0,0, (-2*sqrt(3))*81^(h-1) + (-3)*Ixy_2k(h-1));
check_equal(vector(256,h, Ixy_2k(h)), \
            vector(256,h, Ixy(2*h)));
Ixy_2k(h) = if(h<0,error("h<0"), sum(i=0,h-1, (-2*sqrt(3))*81^(h-1-i)*(-3)^i ));
check_equal(vector(256,h, Ixy_2k(h)), \
            vector(256,h, Ixy(2*h)));
Ixy_2k(h) = if(h<0,error("h<0"), (-2*sqrt(3))* (81^h - (-3)^h)/(81-(-3)) );
check_equal(vector(256,h, Ixy_2k(h)), \
            vector(256,h, Ixy(2*h)));
Ixy_2k(h) = if(h<0,error("h<0"), -sqrt(3)*2/84* (81^h - (-3)^h) );
check_equal(vector(256,h, Ixy_2k(h)), \
            vector(256,h, Ixy(2*h)));

Ixy_k_even(k) = if(k<0,error("k<0"), -sqrt(3)*2/84* (9^k - (-3)^floor(k/2)) );
check_equal(vector(256,h, Ixy_k_even(2*h)), \
            vector(256,h, Ixy(2*h)));

Ixy_k_odd(k) = if(k<0,error("k<0"), -sqrt(3)*2/84* (9^k - (-3/2)*(-3)^floor(k/2)) );
check_equal(vector(256,h, Ixy_k_odd(2*h-1)), \
            vector(256,h, Ixy(2*h-1)));
check_equal(Ixy_k_odd(1), -sqrt(3)/4)

Ixy_k_odd_startxy1(k) = -sqrt(3)*2/84* (9^k - (-3/2)*(-3)^floor(k/2)) + sqrt(3)*1/12*(-3)^floor(k/2);
check_equal(Ixy_k_odd_startxy1(1), Ixy1_with_Iyy1eq1)
Ixy_k_odd_startxy1(k) = -sqrt(3)/168* (4*9^k - 8*(-3)^floor(k/2));
check_equal(Ixy_k_odd_startxy1(1), Ixy1_with_Iyy1eq1)

check_equal(3/2 + (84/-2)*1/12, -2)
\\ 3/2 + (84/-2)*p = -1
\\ p = -5/2*(-2/84) = 5/84
check_equal(3/2 + (84/-2)*5/84, -1)
\\ points factor -1,3/2  segments factor -1,-2

\\ 
\\ Ixy(k) = sqrt(3)/168 * (-4*9^k + [4,-6][(k%2)+1]*(-3)^floor(k/2));

\\ sum Ixx+Iyy == Izz
check_equal(vector(256,k,my(k=k-1); Ixx(k+1) + Iyy(k+1)), \
            vector(256,k,my(k=k-1); 3*Ixx(k) + 3*Iyy(k) + 1/2*9^k));
check_equal(vector(256,k,my(k=k-1); Ixx(k) + Iyy(k)), \
            vector(256,k,my(k=k-1); 1/12 * (9^k - 3^k) ));


\\------------------------------
\\ Istart

\\ 2*m*r^2
\\ 2 * 3^k * (1/2 * sqrt(3)^k * sin(a-deg(60))^2

Istart(k,a) = Ia(k,a-k*deg(30));
\\ Istart(k,a+k*deg(30)) = Ia(k,a);

check_equal(Ia(1, -deg(30)), 3/8)
check_equal(Ia(1,        0), 1/8)
check_equal(Ia(1, deg(120)), 1/2)
check_equal(Ia(1,-deg(120)), 1/8)
check_equal(Istart(1,0),   3/8)

\\ vector(8,k,my(k=k-1); round(Istart(k,0)*8/3))
\\ Istart_next(k,a) = 2*Istart(k,a) + Istart(k,a-deg(120)) + 1/2*9^k*sin(a+deg(60))^2;

check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Ia(k+1,a-(k+1)*deg(30)))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); \
              Ia(k,a-k*deg(30) - deg(120)) \
              + 2*Ia(k,a-k*deg(30)) \
              + 1/2*9^k * (sin(a-k*deg(30)-deg(60)))^2  )))

check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Istart(k+1,a))), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); \
              Istart(k,a - deg(120)) \
              + 2*Istart(k,a) \
              + 1/2*9^k * (sin(a-k*deg(30)-deg(60)))^2  )))

\\ (sin(a -k*deg(30)-deg(60)))^2
\\ = (sin(a)*cos(-k*deg(30)-deg(60)) + cos(a)*sin(-k*deg(30)-deg(60)))^2
\\ = (sin(a)^2        * cos(-k*deg(30)-deg(60))^2 \
\\    + cos(a)^2      * sin(-k*deg(30)-deg(60))^2 \
\\    + sin(a)*cos(a) * cos(-k*deg(30)-deg(60))*sin(-k*deg(30)-deg(60))

\\ [3/4, 1, 3/4, 1/4, 0, 1/4][(k%6)+1]

check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); sin(a -k*deg(30)-deg(60))^2 )), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); \
                sin(a)^2      * [       1/4, 0,       1/4,       3/4, 1,        3/4][(k%6)+1] \
              + cos(a)^2      * [       3/4, 1,       3/4,       1/4, 0,        1/4][(k%6)+1] \
              + sin(a)*cos(a) * [-sqrt(3)/2, 0, sqrt(3)/2, sqrt(3)/2, 0, -sqrt(3)/2][(k%6)+1] )))
