\\ Copyright 2014, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;
deg(x) = x/180*Pi;

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)!="t_VEC",
     if(! nearly_equal(x,y),
        return(Strprintf("different values\n%s\n%s",x,y)));
     return(0));
  if(length(x)!=length(y),
     return(Strprintf("different lengths %s %s",length(x),length(y))));
  for(i=1,#x,
     my(d=differ(x[i],y[i]));
     if(d != 0, return(Strprintf("at index %d %s",i,d))));
  return(0);
}

check_equal(got,want) =
{
  my(d = differ(got,want));
  if(d != 0,
     print("wrong\n",d); quit(1),
     print("ok\n"));
}

\\---------------------------------------
Ia(k,alpha) = Ixx(k)*cos(alpha)^2 + Ixy(k)*cos(alpha)*sin(alpha) + Iyy(k)*sin(alpha)^2;

Ixx(k) =       1/168 * ( 4*9^k - 7*3^k + [3, 6][(k%2)+1]*(-3)^floor(k/2));
Ixy(k) = sqrt(3)/168 * (-4*9^k         + [4,-6][(k%2)+1]*(-3)^floor(k/2));
Iyy(k) =       1/168 * (10*9^k - 7*3^k - [3, 6][(k%2)+1]*(-3)^floor(k/2));

sqrt(3)/2
my(k=14);(Ixx(k)-Iyy(k))/Ixy(k)

check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); \
                (Ixx(k) + Iyy(k))/2 \
                + (Ixx(k) - Iyy(k))/2 * cos(2*a) \
                + Ixy(k)/2            * sin(2*a) \
                )), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-1); Ia(k,a))))

c(k) = Ixy(k)/2  + (Ixx(k) - Iyy(k))/2 * I;
th(k) = arg(c(k));
r(k) = abs(c(k));
check_equal(vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-0); \
                (Ixx(k) + Iyy(k))/2 + r(k) * sin(2*a + th(k)) \
                )), \
            vector(20,a,my(a=2*Pi/a); vector(25,k,my(k=k-0); Ia(k,a))))
abs(c(6))
abs(c(6)^2)
real(c(6)^2)
real(c(6)^2) / abs(c(6)^2)     \\ 1/2 acos 1/7 exactly when k even
real(c(7)^2) / abs(c(7)^2)

\\---------------------------------------
\\ sqrt3 = quadgen(4*3);
\\ sqrt3I = quadgen(4*-3);
\\ sqrt3I = sqrt3*I;
sqrt3 = sqrt(3);
sqrt3I = sqrt(3)*I;
w3 = (-1 + sqrt3I) / 2;  \\ 3rd root of unity, +120deg
w6 =  (1 + sqrt3I) / 2;  \\ 6th root of unity,  +60deg
w12 = (sqrt(3) + I)/2;
b = w6 + 1;
check_equal(abs(w3),1)
check_equal(w12^12,1)
check_equal(w12^6,-1)
check_equal(w3, exp(2*Pi*I/3))
check_equal(w6, exp(2*Pi*I/6))
check_equal(w12, exp(2*Pi*I/12))

rotate_plus120(z) = z*w3;
terdragon_points(k) =
{
  my(e=1,v=[0]);
  for(i=1,k,
     my(p2 = e*w6);
     v = concat(v,
                concat(apply(z->e+rotate_plus120(z), v),
                       apply(z->p2+z,                v)));
     e *= b);
  concat(v,[e]);
}
terdragon_points(1)*2*-w12
check_equal(terdragon_points(0),[0,1])
check_equal(terdragon_points(1),[0,1,w6,w6+1])
  
terdragon_midpoints(k) =
{
  my(v=terdragon_points(k));
  my(m=vector(#v-1,i,0));
  for(i=1,#v-1, m[i] = (v[i]+v[i+1])/2);  \\ average
  m;
}
print("midpoints ",terdragon_midpoints(1)*2);

terdragon_end(k) = b^k;
terdragon_centroid(k) = terdragon_end(k)/2;

Ika_by_sum(k,a) =
{
  my(v=terdragon_midpoints(k));
  my(centroid = terdragon_centroid(k));
  my(rotate = 1/(w12^k * exp(a*I)));
  v = apply(z->(z-centroid)*rotate, v);
  sum(i=1,#v, imag(v[i])^2);
}
Ika_by_sum(1,Pi/2)
Ia(1,Pi/2)
check_equal(vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika_by_sum(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ia(k,a))))

\\----------------------------------------------
Is_by_sum(k,a) =
{
  my(v=terdragon_points(k));
  my(centroid = terdragon_centroid(k));
  my(rotate = 1/(w12^k * exp(a*I)));
  v = apply(z->(z-centroid)*rotate, v);
  my(ret=0);
  for(i=1,#v-1,
      my(m = (v[i]+v[i+1])/2);
      my(l = imag(v[i]-v[i+1]));
      ret += imag(m)^2 + 1/12*l^2);
  ret;
}
Is_by_sum(0,0)
Is_by_sum(0,Pi/2)
Is_by_sum(1,0)
Is_by_sum(1,Pi/2)


\\---------------------------
\\
\\      __*__
\\  __--     --__
\\ *__         __*
\\    --__ __--
\\        *
\\ base b=1
\\ height h/.5=tan30  h=sqrt(3)/6
\\ I triangle = 1/12 b*h^3  for m=1/2*b*h
\\ I triangle = m * 1/6*h^2  for m=1/2*b*h
\\ I two =  2*1/6*(sqrt(3)/6)^2 = 1/36

plot(a=20,30, Ika_by_sum(8,deg(a)))
