#!/usr/bin/perl -w

# Copyright 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Graph;
use Math::PlanePath::AlternateTerdragon;

BEGIN { unshift @INC, $FindBin::Bin }
use MyLineSegments;
use MyPrintwrap ':all';


print "% generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::AlternateTerdragon->new;
my @num_edges;
my @num_vertices;
my @diameters;
foreach my $level (0 .. 7) {

  my ($n_lo, $n_hi) = $path->level_to_n_range($level);

  my $n_end = ($n_hi-1)/2;    # not including middle segment
  my ($x_end,$y_end) = $path->n_to_xy($n_end);

  my $graph = Graph->new (undirected => 1);
  foreach my $n ($n_lo .. $n_end-1) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    $graph->add_edge("$x,$y", "$x2,$y2");
  }
  push @num_vertices, scalar($graph->vertices);
  push @num_edges,    scalar($graph->edges);
  push @diameters,    scalar($graph->diameter) || 0;
}

print "% GP-DEFINE  gen_AltDiameter_half_num_edges = [",
  join(',',@num_edges),"];\n";
print "% GP-DEFINE  gen_AltDiameter_half_num_vertices = [",
  join(',',@num_vertices),"];\n";
print "% GP-DEFINE  gen_AltDiameter_half_diameters = [",
  join(',',@diameters),"];\n";

print "% end generated\n";
exit 0;
