#!/usr/bin/perl -w

# Copyright 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Graph;
use Math::PlanePath::AlternateTerdragon;

BEGIN { unshift @INC, $FindBin::Bin }
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 6;
print "    % generated by tools/$FindBin::Script\n";
print "    \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::AlternateTerdragon->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $n_mid = ($n_hi-1)/2;
my ($x_mid,$y_mid) = $path->n_to_xy($n_mid);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
print "    \\coordinate (end) at ($x_hi,$y_hi);\n";

my $graph = Graph->new (undirected => 1);
my $segments = MyLineSegments->new;

foreach my $n ($n_lo .. $n_mid-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $segments->add($x,$y, $x2,$y2);
  $graph->add_edge("$x,$y", "$x2,$y2");
}

my $diameters = MyLineSegments->new;

my $v_mid = "$x_mid,$y_mid";
my @mid_to_start = $graph->SP_Dijkstra($v_mid, "0,0");
my $want_len = scalar(@mid_to_start);

my $AltDiameterEndpoints_half = 0;
foreach my $v ($graph->vertices) {
  my @vertices = $graph->SP_Dijkstra($v_mid, $v);
  if (@vertices == $want_len) {
    $AltDiameterEndpoints_half++;
    my ($x,$y) = split /,/, $v;
    while ($x != $x_mid || $y != $y_mid) {
      my ($x2,$y2);
      if ($y==0) { ($x2,$y2) = ($x+2,$y); }     # horizontal
      else {
        ($x2,$y2) = ($x+1,$y-1);      # South-East
        if (! $graph->has_edge ("$x,$y", "$x2,$y2")) {
          ($x2,$y2) = ($x-1,$y-1);      # South-West
        }
      }
      $graph->has_edge ("$x,$y", "$x2,$y2")
        or die "oops, no edge $x,$y to $x2,$y2";
      $segments->remove($x,$y, $x2,$y2);
      $diameters->add($x,$y, $x2,$y2);
      ($x,$y) = ($x2,$y2);
    }
  }
}
foreach my $segment ($segments->segments) {
  my ($x1,$y1, $x2,$y2) = @$segment;
  $segments->add($x_hi-$x1,$y_hi-$y1, $x_hi-$x2,$y_hi-$y2);
}
foreach my $segment ($diameters->segments) {
  my ($x1,$y1, $x2,$y2) = @$segment;
  $diameters->add($x_hi-$x1,$y_hi-$y1, $x_hi-$x2,$y_hi-$y2);
}

{
  my ($x_mid2,$y_mid2) = $path->n_to_xy($n_mid+1);
  $diameters->add($x_mid,$y_mid, $x_mid2,$y_mid2);
  if ($segments->is_traversed($x_mid,$y_mid, $x_mid2,$y_mid2)) {
    die;
  }
}

$Printwrap = 4;
printwrap("\\draw[my grey] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 4;
printwrap("\\draw[my thick] ");
$Printwrap += 2;
$diameters->printwrap;
printwrap(";\n");

print "% GP-Test-Last  AltDiameterEndpoints($level) == 2*$AltDiameterEndpoints_half\n";
print "    % end generated\n";
exit 0;
