#!/usr/bin/perl -w

# Copyright 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Graph;
use Math::PlanePath::AlternateTerdragon;

BEGIN { unshift @INC, $FindBin::Bin }
use MyLineSegments;
use MyPrintwrap ':all';
use MyTerdragon '@dir6_to_dx','@dir6_to_dy';


my $level = 5;

print "    % generated by tools/$FindBin::Script\n";
print "    \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::AlternateTerdragon->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
print "    \\coordinate (end) at ($x_hi,$y_hi);\n";

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $segments->add($path->n_to_xy($n), $path->n_to_xy($n+1));
}

my $diameters = MyLineSegments->new;
my $n_mid = ($n_hi-1)/2;
my ($x_mid,$y_mid) = $path->n_to_xy($n_mid);
{
  my ($x_mid2,$y_mid2) = $path->n_to_xy($n_mid+1);
  $segments->remove($x_mid,$y_mid, $x_mid2,$y_mid2);
  $diameters->add($x_mid,$y_mid, $x_mid2,$y_mid2);
}

# my @pending = ([0,0]);
# while (@pending) {
#   ### @pending
#   my @new_pending;
#   my %seen;
#   foreach my $p (@pending) {
#     my ($x,$y) = @$p;
#     ### at: "$x,$y"
# 
#     foreach my $dir6 (0,1) {
#       my $x2 = $x + $dir6_to_dx[$dir6];
#       my $y2 = $y + $dir6_to_dy[$dir6];
#       next if $y <= 4 && $x2-$y2 > 10;
#       my $n = $path->xyxy_to_n_either($x,$y, $x2,$y2);
#       ### to: "$x2,$y2"
#       ### $n
#       if (defined $n && $n <= $n_hi) {
#         $segments->remove($x,$y, $x2,$y2);
#         $diameters->add($x,$y, $x2,$y2);
#         unless ($seen{"$x2,$y2"}++) {
#           push @new_pending, [$x2,$y2];
#         }
#         my $x = $x_hi - $x;
#         my $y = $y_hi - $y;
#         my $x2 = $x_hi - $x2;
#         my $y2 = $y_hi - $y2;
#         $segments->remove($x,$y, $x2,$y2);
#         $diameters->add($x,$y, $x2,$y2);
#       }
#     }
#   }
#   @pending = @new_pending;
# }

{
  my ($x,$y) = (0,0);
  while ($x != $x_mid || $y != $y_mid) {
    my ($x2,$y2);
    if ($x < $x_mid-$y_mid) { ($x2,$y2) = ($x+2,$y); }     # horizontal
    else {  ($x2,$y2) = ($x+1,$y+1); }                     # North-East
    $segments->is_traversed($x,$y, $x2,$y2)
      or die "oops, no edge $x,$y to $x2,$y2";
    $segments->remove($x,$y, $x2,$y2);
    $diameters->add($x,$y, $x2,$y2);
    $segments->remove($x_hi-$x,$y_hi-$y, $x_hi-$x2,$y_hi-$y2);
    $diameters->add($x_hi-$x,$y_hi-$y, $x_hi-$x2,$y_hi-$y2);
    ($x,$y) = ($x2,$y2);
  }
}


$Printwrap = 4;
printwrap("\\draw[my thick] ");
$Printwrap += 2;
$diameters->printwrap;
printwrap(";\n");

$Printwrap = 4;
printwrap("\\draw[my grey] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");


my $graph = Graph->new (undirected => 1);
foreach my $n ($n_lo .. $n_mid-1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $graph->add_edge("$x1,$y1", "$x2,$y2");
}
my $v_mid = "$x_mid,$y_mid";
my @mid_to_start = $graph->SP_Dijkstra($v_mid, "0,0");
my $want_len = scalar(@mid_to_start);

my $AltDiameterEndpoints_half = 0;
foreach my $v ($graph->vertices) {
  my @vertices = $graph->SP_Dijkstra($v_mid, $v);
  if (@vertices == $want_len) {
    $AltDiameterEndpoints_half++;
  }
}

print "% GP-Test-Last  AltDiameterEndpoints($level) == 2*$AltDiameterEndpoints_half\n";
print "    % end generated\n";
exit 0;
