#!/usr/bin/perl -w

# Copyright 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'P1','P2','P7','@dir3_to_dx','@dir3_to_dy',
  'transform_to_horizontal';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 9;
print "  % generated by tools/$FindBin::Script\n";

($level % 4) == 1 or die;

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

print "  \\begin{scope}[xscale={1/6/sqrt(3)},yscale=1/6]\n";

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}
{
  my ($x,$y) = transform($x_hi,$y_hi);
  $y == 0 or die;
}

my ($P1_x,$P1_y) = P1($level);
print "% GP-Test  P1($level) == $P1_x/2 + $P1_y*sqrt3i/2\n";
my $x_min = -7;
my $x_max = 12;
my $y_min = -3;
my $y_max = 3;

# taking untransformed $x,$y
sub in_range {
  my ($x,$y) = @_;
  $x -= $P1_x;
  $y -= $P1_y;
  ($x,$y) = transform($x,$y);
  return ($x_min <= $x && $x <= $x_max
          && $y_min <= $y && $y <= $y_max);
}
in_range($P1_x,$P1_y) or die;

my $segments = MyLineSegments->new;
foreach my $x1 ($P1_x + 5*$x_min .. $P1_x + 5*$x_max) {
  foreach my $y1 ($P1_y + 5*$y_min .. $P1_y + 5*$y_max) {
    foreach my $dir3 (0 .. 2) {
      my $dx = $dir3_to_dx[$dir3];
      my $dy = $dir3_to_dy[$dir3];
      my $x2 = $x1 + $dx;
      my $y2 = $y1 + $dy;
      defined(my $n = $path->xyxy_to_n_either(($x1,$y1, $x2,$y2))) or next;
      $n < $n_hi or next;

      my $in_1 = in_range($x1,$y1);
      my $in_2 = in_range($x2,$y2);
      ($in_1 || $in_2) or next;

      my $x1 = $x1 * 3;
      my $y1 = $y1 * 3;
      $x2 *= 3;
      $y2 *= 3;
      if (! $in_1) {
        $x1 = $x2 - $dx;     # 1/3 of
        $y1 = $y2 - $dy;
      } elsif (! $in_2) {
        $x2 = $x1 + $dx;     # 1/3 of
        $y2 = $y1 + $dy;
      }
      $segments->add($x1,$y1, $x2,$y2);
    }
  }
}

{
  my ($x,$y) = transform($P1_x,$P1_y);
  $x *= 3; $y *= 3;
  print "  \\node (P1) at ($x,$y) [my dot];\n";
}
{
  my ($x,$y) = P2($level);
  ($x,$y) = transform($x,$y);
  $x *= 3; $y *= 3;
  print "  \\coordinate (P2) at ($x,$y);\n";
}
{
  my ($x,$y) = P7($level);
  $x = $x_hi - $x;
  $y = $y_hi - $y;
  ($x,$y) = transform($x,$y);
  $x *= 3; $y *= 3;
  print "  \\coordinate (P7dash) at ($x,$y);\n";
}
$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap (transform => \&transform);
printwrap(";\n");

print "  \\begin{scope}[shift=(end-offset)]\n";
{
  my ($x,$y) = transform(-$P1_x,-$P1_y);
  $x *= 3; $y *= 3;
  print "  \\node (P1dash) at ($x,$y) [my dot];\n";
}
{
  my ($x,$y) = P2($level);
  $x = - $x;
  $y = - $y;
  ($x,$y) = transform($x,$y);
  $x *= 3; $y *= 3;
  print "  \\coordinate (P2dash) at ($x,$y);\n";
}
{
  my ($x,$y) = P7($level);
  $x -= $x_hi;
  $y -= $y_hi;
  ($x,$y) = transform($x,$y);
  $x *= 3; $y *= 3;
  print "  \\coordinate (P7) at ($x,$y);\n";
}
$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap (transform => sub {
                        my ($x,$y) = @_;
                        transform(-$x,-$y);
                      });
printwrap(";\n");
print "  \\end{scope}\n";
print "  \\end{scope}\n";

print "  % end generated\n";
exit 0;
