#!/usr/bin/perl -w

# Copyright 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';

my $level = 3;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my %visits;
my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi - 1) {
  my ($x,$y) = $path->n_to_xy($n);
  $visits{"$x,$y"}++;
  $segments->add($x,$y, $path->n_to_xy($n+1));
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap = 4;
$segments->printwrap;
printwrap(";\n");

my $TB = 0;
$Printwrap = 2;
foreach my $xy (sort keys %visits) {
  if ($visits{$xy} == 3) {
    $TB++;
    printwrap(" \\node at ($xy) [my dot];");
  }
}
printwrap_newline();

print "  \\coordinate (start) at (0,0);\n";
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  print "  \\coordinate (end) at ($x,$y);\n";
}

print "% GP-Test-Last  TB($level) == $TB\n";
print "  % end generated\n";
exit 0;
