#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::AlternateTerdragon;

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyPrintwrap ':all';


my $max_level = 4;
print "  % generated by tools/$FindBin::Script\n";

if (@ARGV) {
  $max_level = shift @ARGV;
}

my $path = Math::PlanePath::AlternateTerdragon->new;

sub is_touch {
  my ($n, $n_hi) = @_;
  my ($x,$y) = $path->n_to_xy($n);
  foreach my $n_other ($path->xy_to_n_list($x,$y)) {
    next if $n_other == $n;
    next if $n_other > $n_hi;
    return 1;
  }
  return 0;
}

sub level_any_touch {
  my ($level) = @_;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  foreach my $n ($n_lo .. $n_hi) {
    if (is_touch($n, $n_hi)) {
      return 1;
    }
  }
  return 0;
}

foreach my $level (0 .. $max_level) {
  $Printwrap = 2;
  printwrap("\\begin{scope}[shift=(pos-$level)]\n");
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);

  {
    my ($x,$y) = $path->n_to_xy($n_hi);
    print "    \\node (start-$level) at (0,0) [my dot];\n";
    print "    \\node (end-$level) at ($x,$y) [my dot];\n";
  }

  $Printwrap = 4;
  printwrap("\\draw[my line] (start-$level)");
  my $any_touch = level_any_touch($level);
  foreach my $n ($n_lo+1 .. $n_hi-1) {
    my ($x,$y) = $path->n_to_xy($n);
    if ($any_touch) {
      my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
      my ($tdx,$tdy) = $path->n_to_dxdy($n);
      my $sx = $x . chamfer(-$sdx);
      my $sy = $y . chamfer(-$sdy);
      my $tx = $x . chamfer($tdx);
      my $ty = $y . chamfer($tdy);
      printwrap("--($sx,$sy)");
      printwrap("--($tx,$ty)");
    } else {
      printwrap("--($x,$y)");
    }
  }
  printwrap("--(end-$level);\n");
  $Printwrap = 2;
  printwrap("\\end{scope}\n");
}
print "  % end generated\n";
exit 0;
