#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use List::Util 'min','max';
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


print "  % generated by tools/$FindBin::Script\n";

my $level = 2;

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);


my %points = ("$x_hi,{$y_hi*sqrt(3)}" => 1);
my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  $points{"$x,{$y*sqrt(3)}"} = 1;
  $segments->add($x,$y, $path->n_to_xy($n+1));
}
### $segments

sub stringize {
  my ($x,$y) = @_;
  return "($x,{$y*sqrt(3)})";
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap(stringize => \&stringize);
printwrap(";\n");

$Printwrap = 2;
foreach my $point (sort keys %points) {
  printwrap(" \\filldraw ($point)circle;");
}
printwrap_newline();

foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  printwrap(" \\coordinate (p$n) at ($x,{$y*sqrt(3)});");
}
printwrap_newline();

print "  % end generated\n";
exit 0;
