#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyPrintwrap ':all';


$Printwrap = 2;
printwrap("% generated by tools/$FindBin::Script\n");

my $level = 11;
my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
my $ex = 2;
my $ey = 0;

my $points = [
              [0,0],
              [$ex,$ey],
             ];

foreach (1 .. $level) {
  ($points,$ex,$ey) = hull_expand($points,$ex,$ey);
  # print "  % count ",scalar(@$points),"\n";
}

my $x_offset = 0;
my $y_offset = 0;
my $rotate = 0;
my $type = '';
sub draw {
  printwrap("\\draw[$type] ");
  my $join = '';
  $Printwrap = 2;
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    if ($rotate) {
      ($x,$y) = xy_rotate_plus120($x,$y);
    }
    $x += $x_offset;
    $y += $y_offset;
    $x /= 100;
    $y /= 100;
    printwrap("$join($x,$y) node[my dot]\n");
    $join = '--';
  }
  printwrap("--cycle;\n");
}

draw();

($x_offset,$y_offset) = ($x_hi,$y_hi);
$rotate = 1;
draw();

($x_offset,$y_offset) = $path->n_to_xy(2*$n_hi);
$rotate = 0;
draw();

$x_hi /= 100; $y_hi /= 100;
print "  \\node (M1) at ($x_hi,$y_hi) [my dot];\n";

($n_lo, $n_hi) = $path->level_to_n_range($level+1);
($x_hi,$y_hi) = $path->n_to_xy($n_hi);
$x_hi /= 100; $y_hi /= 100;
print "  \\node (end) at ($x_hi,$y_hi) [my dot];\n";


# my ($next_x_hi,$next_y_hi) = $path->n_to_xy($next_n_hi);
# $x_offset = $next_x_hi - $x_hi;
# $y_offset = $next_y_hi - $y_hi;
# 
# $type = 'dashed';
# ($points,$ex,$ey) = hull_expand($points,$ex,$ey);
# $x_offset = 0;
# $y_offset = 0;
# $rotate = 0;
# draw();


# foreach my $p (@$points) {
#   my ($x,$y) = @$p;
#   ($x,$y) = mul($x,$y);
#   $p = [$x,$y]; # mutate
# }
# $x_offset = 0;
# $y_offset = 0;
# $rotate = 0;
# $type = 'dashed';
# draw();

# @points = ([-84,-30],[-80,-34],[-71,-37],[-51,-37],[-21,-27],[-17,-23],[3,1],[3,3],[-1,7],[-10,10],[-30,10],[-60,0],[-64,-4],[-84,-28]);
# $x_offset = 0;
# $y_offset = 0;
# $rotate = 0;
# $type = 'dashed';
# draw();


print "  % end generated\n";
exit 0;
