#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";
foreach my $level (0, 1) {
  my $arms = 6;
  my $path = Math::PlanePath::TerdragonCurve->new (arms => $arms);
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);

  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo + $arms .. $n_hi) {
    $segments->add($path->n_to_xy($n),
                   $path->n_to_xy($n - $arms));
  }

  $Printwrap = 2;
  if ($level) {
    printwrap("\\begin{scope}[shift=(right-pos)]\n");
    $Printwrap = 4;
  }
  printwrap("% arms=$arms N=$n_lo to $n_hi\n");
  printwrap("\\filldraw (0,0) circle;\n");
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");

  if ($level) {
    $Printwrap = 2;
    printwrap("\\end{scope}\n");
  }
}

print "  % end generated\n";
exit 0;
