#!/usr/bin/perl -w

# Copyright 2014, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonRounded 117; # v.117 for level_to_n_range()
use Math::NumSeq::PlanePathTurn;

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon;
use MyPrintwrap ':all';


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::TerdragonRounded->new;
my $seq = Math::NumSeq::PlanePathTurn->new(planepath_object => $path,
                                           turn_type => 'LSR');

my ($n_lo, $n_hi) = $path->level_to_n_range($level);

print "  % n_hi = $n_hi\n";
my $n_start = 2*($n_hi+1);
my $n_end = 5*($n_hi+1);
my ($ox,$oy) = $path->n_to_xy($n_start);

{
  $Printwrap = 2;
  printwrap("\\draw[my line] ");
  my $join = '';
  foreach my $n ($n_start .. $n_end-1) {
    my ($x,$y) = $path->n_to_xy($n);
    $x -= $ox;
    $y -= $oy;
    printwrap("$join($x,{$y*sqrt(3)})");
    $join = '--';
  }
  printwrap("--cycle;\n");
}

{
  $Printwrap = 2;
  foreach my $n ($n_start .. $n_end-1) {
    my $turn = ($n==$n_start || $n==$n_end-1 ? 1 : $seq->ith($n));
    next unless $turn == -1;
    my ($x,$y) = $path->n_to_xy($n);
    $x -= $ox;
    $y -= $oy;
    printwrap(" \\node at ($x,{$y*sqrt(3)}) [my dot];");
  }
  printwrap_newline();
}

print "  % end generated\n";
exit 0;
