#!/usr/bin/perl -w

# Copyright 2014, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Lingua::EN::Numbers;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyPrintwrap ':all';


my $level = 2;

$Printwrap = 2;
printwrap("% generated by tools/$FindBin::Script\n");
printwrap("\\newcommand{\\MyLevel}{$level}\n");

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);


sub rotate {
  my ($x,$y) = @_;
  foreach (1 .. $level) {
    ($x,$y) = xy_rotate_minus120($x,$y);
  }
  return ($x,$y);
}

my $join = '';
my @count = (0,0,0);
my $x_max_x = 0;
my $x_max_y = 0;
foreach my $n ($n_lo .. $n_hi-1) {
  my $end = ($n == $n_hi-1 ? ';' : '');
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);

  ($x,$y) = rotate($x,$y);
  ($dx,$dy) = rotate($dx,$dy);
  my $x2 = $x + $dx;
  my $y2 = $y + $dy;

  printwrap("\\draw [->,my shorten] ($x,{$y*sqrt(3)}) -- ($x2,{$y2*sqrt(3)});");
  $join = '--';

  if ($n != $n_hi) {
    my $dir3 = dxdy_to_dir3($dx,$dy);
    $count[$dir3]++;
  }

  if ($x > $x_max_x) {
    $x_max_x = $x;
    $x_max_y = $y;
  }
}
printwrap_newline();

foreach my $i (0 .. $#count) {
  my $name = Lingua::EN::Numbers::num2en($i);
  printwrap("\\newcommand{\\MyCount\u$name}{$count[$i]}\n");
}
foreach my $i (0 .. $#count) {
  print "% GP-Test  SM($level,$i) == $count[$i]\n";
}

{
  my $n = ($n_hi+1)/2; # middle segment end
  my ($x,$y) = $path->n_to_xy($n);
  ($x,$y) = rotate($x,$y);
  printwrap("\\coordinate (middle) at ($x,{$y*sqrt(3)});\n");
}
{
  my $n = $n_hi; # middle segment end
  my ($x,$y) = $path->n_to_xy($n);
  ($x,$y) = rotate($x,$y);
  printwrap("\\coordinate (end) at ($x,{$y*sqrt(3)});\n");
}
printwrap("\\coordinate (X-max) at ($x_max_x,{$x_max_y*sqrt(3)});\n");

printwrap("% end generated\n");
exit 0;
