#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 3;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
my $rsh = 2**$level;
my $jr = 2**($level-1);

my $x = 0;
my $y = 0;
my $dx = 2;
my $dy = 0;
my @points;
foreach my $i (1 .. 2**$level) {
  push @points, [$x,$y];
  $x += $dx;
  $y += $dy;
  if (BitAboveLowestOne($i)) {
    ($dx,$dy) = xy_rotate_minus60($dx,$dy);
  } else {
    ($dx,$dy) = xy_rotate_plus60($dx,$dy);
  }
}
my $x_end = $x;
my $y_end = $y;
my ($xp2,$yp2) = xy_rotate_plus60($x_end,$y_end);

foreach my $i (0 .. $#points) {
  my ($x,$y) = @{$points[$i]};
  push @points, [$x_end - $x, $y_end - $y];
}
### @points
### end: "$x_end,$y_end   p2=$xp2,$yp2"

my @points1 = map { my ($x,$y) = @$_;
                    ($x,$y) = xy_rotate_plus120($x,$y);
                    [ $x_end + $x, $y_end + $y ]
                  } @points;

my @points2 = map { my ($x,$y) = @$_;
                    [ $xp2 + $x, $yp2 + $y ]
                  } @points;

{
my $join = '';
$Printwrap = 2;
printwrap("\\draw ");
$Printwrap = 4;
foreach my $i (0 .. $rsh-1) {   # part0 right
  my $p = $points[$i];
  my ($x,$y) = @$p;
  printwrap("$join($x,{$y*sqrt(3)})");
  $join = '--';
}
foreach my $i (0 .. $jr-1) {   # part1 right
  my $p = $points1[$i];
  my ($x,$y) = @$p;
  printwrap("$join($x,{$y*sqrt(3)})");
  $join = '--';
}
foreach my $i ($jr .. 2*$rsh-1) {   # part2 right and left
  my $p = $points2[$i];
  my ($x,$y) = @$p;
  printwrap("$join($x,{$y*sqrt(3)})");
  $join = '--';
}
foreach my $i ($rsh .. $rsh+$jr-1) {   # part1 left
  my $p = $points1[$i];
  my ($x,$y) = @$p;
  printwrap("$join($x,{$y*sqrt(3)})");
  $join = '--';
}
foreach my $i ($rsh+$jr .. 2*$rsh-1) {   # part0 left
  my $p = $points[$i];
  my ($x,$y) = @$p;
  printwrap("$join($x,{$y*sqrt(3)})");
  $join = '--';
}
printwrap("--cycle;\n");
}

my $join_length = $jr;
{
  my $join = '';
  $Printwrap = 2;
  printwrap("\\draw[very thick] ");
  $Printwrap = 4;
  foreach my $i ($rsh .. $rsh+$jr) {   # join 0 to 1
    my $p = $points[$i];
    my ($x,$y) = @$p;
    printwrap("$join($x,{$y*sqrt(3)})");
    $join = '--';
  }
  printwrap(";\n");
}
{
  my $join = '';
  $Printwrap = 2;
  printwrap("\\draw[very thick] ");
  $Printwrap = 4;
  foreach my $i ($jr .. $rsh) {   # join 1 to 2
    my $p = $points1[$i];
    my ($x,$y) = @$p;
    printwrap("$join($x,{$y*sqrt(3)})");
    $join = '--';
  }
  printwrap(";\n");
}

$Printwrap = 2;
foreach my $p ($points[0],
               $points[$rsh],
               $points2[0],
               $points2[$rsh]) {
  my ($x,$y) = @$p;
  printwrap("\\fill ($x,{$y*sqrt(3)})circle;\n");
}

print "  \\newcommand{\\MyJBSH}{$join_length}\n";
print "% GP-Test-Last  JBSH($level)==$join_length\n";
print "  % end generated\n";
exit 0;














# {
#   my $join = '';
#   $Printwrap = 2;
#   printwrap("\\draw ");
#   $Printwrap = 4;
#   foreach my $p (@points1) {
#     my ($x,$y) = @$p;
#     printwrap("$join($x,{$y*sqrt(3)})");
#     $join = '--';
#   }
#   printwrap("--cycle;\n");
# }
# {
#   my $join = '';
#   $Printwrap = 2;
#   printwrap("\\draw ");
#   $Printwrap = 4;
#   foreach my $p (@points2) {
#     my ($x,$y) = @$p;
#     printwrap("$join($x,{$y*sqrt(3)})");
#     $join = '--';
#   }
#   printwrap("--cycle;\n");
# }
